@@ -67,8 +67,7 @@ struct TwoDimList *list;
 
 { static char entry[bufsize];
   struct TwoDimList *tp;
-  struct Item *ip;
-  char seps[2], *sp;
+  char seps[2];
 
 Debug1("Get2DListEnt()\n");
 
@@ -121,7 +120,7 @@ if ((strlen(varstring) == 1) && (varstring[0] == sep))
    return;
    }
 
-sprintf(format,"%%[^%c]",sep);   /* set format string to search */
+snprintf(format,6,"%%[^%c]",sep);   /* set format string to search */
 
 for (sp = varstring; *sp != '\0'; sp++)
    {
@@ -252,6 +251,12 @@ char sep;
 
 Debug1("AppendTwoDimItem(itemlist, sep=%c)\n",sep);
 
+ if (liststart == NULL)
+    {
+    Debug("SOFTWARE ERROR in AppendTwoDimItem()\n ");
+    return;
+    }
+ 
 if ((ip = (struct TwoDimList *)malloc(sizeof(struct TwoDimList))) == NULL)
    {
    CfLog(cferror,"AppendTwoDimItem","malloc");
