@@ -34,19 +34,18 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
-CompareMD5CheckSums(file1,file2,ip,sstat,dstat)
+/*******************************************************************/
 
-     /* See the md5 algorithms in pub-lib/md5.c */
-     /* file 1 is source                        */
+int CompareCheckSums(file1,file2,ip,sstat,dstat)
 
 char *file1, *file2;
 struct Image *ip;
 struct stat *sstat, *dstat;
 
- { static unsigned char digest1[16], digest2[16];
+{ static unsigned char digest1[EVP_MAX_MD_SIZE+1], digest2[EVP_MAX_MD_SIZE+1];
   int i;
 
-Debug("CompareMD5CheckSums(%s,%s)\n",file1,file2);
+Debug("CompareCheckSums(%s,%s)\n",file1,file2);
 
 if (sstat->st_size != dstat->st_size)
    {
@@ -58,18 +57,19 @@ Debug2("Compare checksums on %s:%s & %s\n",ip->server,file1,file2);
 
 if (strcmp(ip->server,"localhost") == 0)
    {
-   cfMDFile(file1,digest1);
-   cfMDFile(file2,digest2);
+   ChecksumFile(file1,digest1,'m');
+   ChecksumFile(file2,digest2,'m');
 
-   for (i = 0; i < 16; i++)
+   for (i = 0; i < EVP_MAX_MD_SIZE; i++)
       {
       if (digest1[i] != digest2[i])
          {
-         Verbose("Checksum (MD5) mismatch...\n");
+         Verbose("Checksum mismatch...\n");
          return true;
          }
       }
 
+   Debug("Files were identical\n");
    return false;  /* only if files are identical */
    }
 else
@@ -80,7 +80,7 @@ else
 
 /*******************************************************************/
 
-CompareBinarySums(file1,file2,ip,sstat,dstat)
+int CompareBinarySums(file1,file2,ip,sstat,dstat)
 
      /* See the md5 algorithms in pub-lib/md5.c */
      /* file 1 is source                        */
@@ -89,9 +89,8 @@ char *file1, *file2;
 struct Image *ip;
 struct stat *sstat, *dstat;
 
- { static unsigned char digest1[16], digest2[16];
-   int i,fd1, fd2,bytes1, bytes2;
-   char	buff1[BUFSIZ], buff2[BUFSIZ];
+ { int fd1, fd2,bytes1,bytes2;
+   char	buff1[BUFSIZ],buff2[BUFSIZ];
 
 Debug("CompareBinarySums(%s,%s)\n",file1,file2);
 
@@ -105,8 +104,8 @@ Debug2("Compare binary sums on %s:%s & %s\n",ip->server,file1,file2);
 
 if (strcmp(ip->server,"localhost") == 0)
    {
-   fd1 = open(file1, O_RDONLY, 0400);
-   fd2 = open(file2, O_RDONLY, 0400);
+   fd1 = open(file1, O_RDONLY|O_BINARY, 0400);
+   fd2 = open(file2, O_RDONLY|O_BINARY, 0400);
   
    do
       {
@@ -130,8 +129,107 @@ if (strcmp(ip->server,"localhost") == 0)
    }
 else
    {
-   Verbose("Using cached md5 checksum instead\n");
+   Debug("Using network md5 checksum instead\n");
    return CompareMD5Net(file1,file2,ip); /* client.c */
    }
 }
 
+/*******************************************************************/
+
+void ChecksumFile(filename,digest,type)
+
+char *filename,type;
+unsigned char digest[EVP_MAX_MD_SIZE+1];
+
+{ FILE *file;
+  EVP_MD_CTX context;
+  int len, md_len;
+  unsigned char buffer[1024];
+  const EVP_MD *md = NULL;
+
+Debug2("ChecksumFile(%c,%s)\n",type,filename);
+
+if ((file = fopen (filename, "rb")) == NULL)
+   {
+   printf ("%s can't be opened\n", filename);
+   }
+else
+   {
+   switch (type)
+      {
+      case 's': md = EVP_get_digestbyname("sha");
+  	        break;
+      case 'm': md = EVP_get_digestbyname("md5");
+	        break;
+      default: FatalError("Software failure in ChecksumFile");
+      }
+   
+   EVP_DigestInit(&context,md);
+
+   while (len = fread(buffer,1,1024,file))
+      {
+      EVP_DigestUpdate(&context,buffer,len);
+      }
+
+   EVP_DigestFinal(&context,digest,&md_len);
+   
+   /* Digest length stored in md_len */
+   fclose (file);
+   }
+}
+
+/*******************************************************************/
+
+void ChecksumString(buffer,len,digest,type)
+
+char *buffer,type;
+int len;
+unsigned char digest[EVP_MAX_MD_SIZE+1];
+
+{ EVP_MD_CTX context;
+  const EVP_MD *md = NULL;
+  int md_len;
+
+Debug2("ChecksumString(%c)\n",type);
+
+switch (type)
+   {
+   case 's': md = EVP_get_digestbyname("sha");
+       break;
+   case 'm': md = EVP_get_digestbyname("md5");
+       break;
+   default: FatalError("Software failure in ChecksumFile");
+   }
+ 
+EVP_DigestInit(&context,md); 
+EVP_DigestUpdate(&context,(unsigned char*)buffer,len);
+EVP_DigestFinal(&context,digest,&md_len);
+}
+
+/*******************************************************************/
+
+int ChecksumsMatch(digest1,digest2,type)
+
+unsigned char digest1[EVP_MAX_MD_SIZE+1],digest2[EVP_MAX_MD_SIZE+1];
+char type;
+
+{ int i,size = EVP_MAX_MD_SIZE;
+ 
+switch(type)  /* Size of message digests */
+   {
+   case 'm': size = CF_MD5_LEN;
+       break;
+   case 's': size = CF_SHA_LEN;
+       break;
+   }
+
+for (i = 0; i < size; i++)
+   {
+   if (digest1[i] != digest2[i])
+      {
+      return false;
+      }
+   }
+
+return true;
+}
