@@ -28,9 +28,9 @@
 /*                                                                 */
 /*******************************************************************/
 
+#include "../pub/getopt.h"
 #include "cf.defs.h"
 #include "cf.extern.h"
-#include "../pub/getopt.h"
 
 #ifdef HAVE_SYS_STATFS_H
  #include <sys/statfs.h>
@@ -69,28 +69,28 @@ enum cfsizes type;
 #if defined ULTRIX
     if (getmnt (NULL, &buf, sizeof (struct fs_data), STAT_ONE, file) == -1)
        {
-       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       snprintf(OUTPUT,bufsize,"Couldn't get filesystem info for %s\n",file);
        CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #elif defined SOLARIS || defined OSF || defined UNIXWARE 
     if (statvfs (file, &buf) != 0)
        {
-       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       snprintf(OUTPUT,bufsize,"Couldn't get filesystem info for %s\n",file);
        CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #elif defined IRIX || defined SCO || defined CFCRAY || defined UNIXWARE
     if (statfs (file, &buf, sizeof (struct statfs), 0) != 0)
        {
-       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       snprintf(OUTPUT,bufsize,"Couldn't get filesystem info for %s\n",file);
        CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #else
     if (statfs (file, &buf) != 0)
        {
-       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       snprintf(OUTPUT,bufsize,"Couldn't get filesystem info for %s\n",file);
        CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
@@ -108,7 +108,7 @@ enum cfsizes type;
     avail = buf.f_bavail * (buf.f_frsize / blocksize);
 #endif
 
-#if defined NETBSD || defined FREEBSD || defined OPENBSD || defined SUNOS || defined HPuUX
+#if defined NETBSD || defined FREEBSD || defined OPENBSD || defined SUNOS || defined HPuUX || defined DARWIN
     total = buf.f_blocks;
     used = buf.f_blocks - buf.f_bfree;
     avail = buf.f_bavail;
