@@ -70,7 +70,7 @@ char *s;
   char *p;
 
 v = 0;
-while (isdigit(*s))
+while (isdigit((int)*s))
    {
    v = 10*v + *s++ - '0';
    }
